namespace Telerik.Reporting.Examples.CSharp
{
    using System;
    using System.ComponentModel;
    using System.Drawing;
    using System.Windows.Forms;
    using Telerik.Reporting;
    using Telerik.Reporting.Drawing;
using System.Collections.Generic;
    using Telerik.Reporting.Expressions;

    /// <summary>
    /// Summary description for ProductSales.
    /// </summary>
    [Description("Crosstab summary of AdventureWorks sales, grouped by product category over a four-year period")]
    public partial class ProductSales : Telerik.Reporting.Report
    {
        public ProductSales()
        {
            /// <summary>
            /// Required for telerik Reporting designer support
            /// </summary>
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //
        }

        public static int Qtr(DateTime date)
        {
            return (1 + ((date.Month - 1) / 3));
        }
    }


    /// <summary>
    /// Defines an aggregate function to sum numeric values separated by some key.
    /// </summary>
    [AggregateFunction(Description="Defines an aggregate function to sum numeric values separated by some key.")]
    /// a certain boolean filtering condition.
    public class SlotsSum : IAggregateFunction
    {
        // Contains the currently accumulated values of the aggregate function.
        SumsDictionary sums;

        /// <summary>
        /// Initializes the current aggregate function to its initial state
        /// ready to accumulate and merge values. 
        /// </summary>
        public void Init()
        {
            this.sums = new SumsDictionary();
        }

        /// <summary>
        /// Accumulates new argument values to the current aggregate function.
        /// </summary>
        /// <param name="values">
        /// Specifies an array of argument values for the aggregate function.
        /// </param>
        public void Accumulate(object[] values)
        {
            // Extract the number argument from the specified array.
            double number = Convert.ToDouble(values[0]);
            // Extract the key argument from the specified array.
            object key = values[1];

            // Accumulate each value in its slot.
            double currentValue;
            if(this.sums.TryGetValue(key, out currentValue))
            {
                this.sums[key] = currentValue + number;
            }
            else
            {
                this.sums[key] = number;
            }
        }

        /// <summary>
        /// Merges the specified aggregate function to the current one.
        /// </summary>
        /// <param name="aggregate">
        /// Specifies an aggregate function to be merged to the current one.
        /// </param>
        /// <remarks>
        /// This method allows the reporting engine to merge two accumulated
        /// subsets of the same aggregate function into a single result.
        /// </remarks>
        public void Merge(IAggregateFunction aggregate)
        {
            // Accumulate the values of the specified aggregate function.
            Dictionary<object, double> sums1 = (Dictionary<object, double>)aggregate.GetValue();
            foreach (object key in sums1.Keys)
            {
                this.Accumulate(new object[] { sums1[key], key });
            }
        }

        /// <summary>
        /// Returns the currently accumulated value of the aggregate function.
        /// </summary>
        /// <returns>
        /// The currently accumulated numeric value of the aggregate function.
        /// </returns>
        public object GetValue()
        {
            return this.sums;
        }
    }

    /// <summary>
    /// Dedicated (object, double) dictionary providing GetValue functionality
    /// that will be used from expressions.
    /// </summary>
    class SumsDictionary : Dictionary<object, double>
    {
        public object GetValue(object key)
        {
            if (null == key)
            {
                return null;
            }

            double value;
            if (this.TryGetValue(key, out value))
            {
                return value;
            }
            else
            {
                return null;
            }
        }
    }
}